@@ -4,12 +4,17 @@ use strict;
 use warnings;
 use namespace::autoclean 0.19 -except => ['import'];
 
+use DateTime::Locale::Data;
+
 use Exporter qw( import );
 
-our $VERSION = '1.22';
+our $VERSION = '1.40';
 
 our @EXPORT_OK = 'parse_locale_code';
 
+# This could probably all be done in a cleaner way starting with _just_
+# checking the known codes first and only then falling back to heuristics. But
+# for now it's good enough to handle oddballs like be-tarask and en-polyton.
 sub parse_locale_code {
     my @pieces = split /-/, $_[0];
 
@@ -17,7 +22,21 @@ sub parse_locale_code {
 
     my %codes = ( language => lc shift @pieces );
     if ( @pieces == 1 ) {
-        if ( length $pieces[0] == 2 || $pieces[0] =~ /^\d\d\d$/ ) {
+        ## no critic (ControlStructures::ProhibitCascadingIfElse, Variables::ProhibitPackageVars)
+        if ( exists $DateTime::Locale::Data::VariantCodes{ uc $pieces[0] } ) {
+            $codes{variant} = uc shift @pieces;
+        }
+        elsif (
+            exists $DateTime::Locale::Data::TerritoryCodes{ uc $pieces[0] } )
+        {
+            $codes{territory} = uc shift @pieces;
+        }
+        elsif (
+            exists $DateTime::Locale::Data::ScriptCodes{ _tc( $pieces[0] ) } )
+        {
+            $codes{script} = _tc( shift @pieces );
+        }
+        elsif ( length $pieces[0] == 2 || $pieces[0] =~ /^\d\d\d$/ ) {
             $codes{territory} = uc shift @pieces;
         }
         else {
@@ -66,7 +85,7 @@ DateTime::Locale::Util - Utility code for DateTime::Locale
 
 =head1 VERSION
 
-version 1.22
+version 1.40
 
 =head1 DESCRIPTION
 
@@ -79,8 +98,6 @@ Bugs may be submitted at L<https://github.com/houseabsolute/DateTime-Locale/issu
 There is a mailing list available for users of this distribution,
 L<mailto:datetime@perl.org>.
 
-I am also usually active on IRC as 'autarch' on C<irc://irc.perl.org>.
-
 =head1 SOURCE
 
 The source code repository for DateTime-Locale can be found at L<https://github.com/houseabsolute/DateTime-Locale>.
@@ -91,7 +108,7 @@ Dave Rolsky <autarch@urth.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2003 - 2018 by Dave Rolsky.
+This software is copyright (c) 2003 - 2023 by Dave Rolsky.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
